"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var core = _interopRequireWildcard(require("../../core"));

var _generateBlurVertSource = _interopRequireDefault(require("./generateBlurVertSource"));

var _generateBlurFragSource = _interopRequireDefault(require("./generateBlurFragSource"));

var _getMaxBlurKernelSize = _interopRequireDefault(require("./getMaxBlurKernelSize"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

/**
 * The BlurYFilter applies a horizontal Gaussian blur to an object.
 *
 * @class
 * @extends PISI.Filter
 * @memberof PISI.filters
 */
var BlurYFilter = /*#__PURE__*/function (_core$Filter) {
  _inherits(BlurYFilter, _core$Filter);

  var _super = _createSuper(BlurYFilter);

  /**
   * @param {number} strength - The strength of the blur filter.
   * @param {number} quality - The quality of the blur filter.
   * @param {number} resolution - The resolution of the blur filter.
   * @param {number} [kernelSize=5] - The kernelSize of the blur filter.Options: 5, 7, 9, 11, 13, 15.
   */
  function BlurYFilter(strength, quality, resolution, kernelSize) {
    var _this;

    _classCallCheck(this, BlurYFilter);

    kernelSize = kernelSize || 5;
    var vertSrc = (0, _generateBlurVertSource["default"])(kernelSize, false);
    var fragSrc = (0, _generateBlurFragSource["default"])(kernelSize);
    _this = _super.call(this, // vertex shader
    vertSrc, // fragment shader
    fragSrc);
    _this.resolution = resolution || core.settings.RESOLUTION;
    _this._quality = 0;
    _this.quality = quality || 4;
    _this.strength = strength || 8;
    _this.firstRun = true;
    return _this;
  }
  /**
   * Applies the filter.
   *
   * @param {PISI.FilterManager} filterManager - The manager.
   * @param {PISI.RenderTarget} input - The input target.
   * @param {PISI.RenderTarget} output - The output target.
   * @param {boolean} clear - Should the output be cleared before rendering?
   */


  _createClass(BlurYFilter, [{
    key: "apply",
    value: function apply(filterManager, input, output, clear) {
      if (this.firstRun) {
        var gl = filterManager.renderer.gl;
        var kernelSize = (0, _getMaxBlurKernelSize["default"])(gl);
        this.vertexSrc = (0, _generateBlurVertSource["default"])(kernelSize, false);
        this.fragmentSrc = (0, _generateBlurFragSource["default"])(kernelSize);
        this.firstRun = false;
      }

      this.uniforms.strength = 1 / output.size.height * (output.size.height / input.size.height);
      this.uniforms.strength *= this.strength;
      this.uniforms.strength /= this.passes;

      if (this.passes === 1) {
        filterManager.applyFilter(this, input, output, clear);
      } else {
        var renderTarget = filterManager.getRenderTarget(true);
        var flip = input;
        var flop = renderTarget;

        for (var i = 0; i < this.passes - 1; i++) {
          filterManager.applyFilter(this, flip, flop, true);
          var temp = flop;
          flop = flip;
          flip = temp;
        }

        filterManager.applyFilter(this, flip, output, clear);
        filterManager.returnRenderTarget(renderTarget);
      }
    }
    /**
     * Sets the strength of both the blur.
     *
     * @member {number}
     * @default 2
     */

  }, {
    key: "blur",
    get: function get() {
      return this.strength;
    },
    set: function set(value) {
      this.padding = Math.abs(value) * 2;
      this.strength = value;
    }
    /**
     * Sets the quality of the blur by modifying the number of passes. More passes means higher
     * quaility bluring but the lower the performance.
     *
     * @member {number}
     * @default 4
     */

  }, {
    key: "quality",
    get: function get() {
      return this._quality;
    },
    set: function set(value) {
      this._quality = value;
      this.passes = value;
    }
  }]);

  return BlurYFilter;
}(core.Filter);

exports["default"] = BlurYFilter;
//# sourceMappingURL=BlurYFilter.js.map