"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _pixiGlCore = _interopRequireDefault(require("pixi-gl-core"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * An object containing WebGL specific properties to be used by the WebGL renderer
 *
 * @class
 * @private
 * @memberof PISI
 */
var WebGLGraphicsData = /*#__PURE__*/function () {
  /**
   * @param {WebGLRenderingContext} gl - The current WebGL drawing context
   * @param {PISI.Shader} shader - The shader
   * @param {object} attribsState - The state for the VAO
   */
  function WebGLGraphicsData(gl, shader, attribsState) {
    _classCallCheck(this, WebGLGraphicsData);

    /**
     * The current WebGL drawing context
     *
     * @member {WebGLRenderingContext}
     */
    this.gl = gl; // TODO does this need to be split before uploading??

    /**
     * An array of color components (r,g,b)
     * @member {number[]}
     */

    this.color = [0, 0, 0]; // color split!

    /**
     * An array of points to draw
     * @member {PISI.Point[]}
     */

    this.points = [];
    /**
     * The indices of the vertices
     * @member {number[]}
     */

    this.indices = [];
    /**
     * The main buffer
     * @member {WebGLBuffer}
     */

    this.buffer = _pixiGlCore["default"].GLBuffer.createVertexBuffer(gl);
    /**
     * The index buffer
     * @member {WebGLBuffer}
     */

    this.indexBuffer = _pixiGlCore["default"].GLBuffer.createIndexBuffer(gl);
    /**
     * Whether this graphics is dirty or not
     * @member {boolean}
     */

    this.dirty = true;
    /**
     * Whether this graphics is nativeLines or not
     * @member {boolean}
     */

    this.nativeLines = false;
    this.glPoints = null;
    this.glIndices = null;
    /**
     *
     * @member {PISI.Shader}
     */

    this.shader = shader;
    this.vao = new _pixiGlCore["default"].VertexArrayObject(gl, attribsState).addIndex(this.indexBuffer).addAttribute(this.buffer, shader.attributes.aVertexPosition, gl.FLOAT, false, 4 * 6, 0).addAttribute(this.buffer, shader.attributes.aColor, gl.FLOAT, false, 4 * 6, 2 * 4);
  }
  /**
   * Resets the vertices and the indices
   */


  _createClass(WebGLGraphicsData, [{
    key: "reset",
    value: function reset() {
      this.points.length = 0;
      this.indices.length = 0;
    }
    /**
     * Binds the buffers and uploads the data
     */

  }, {
    key: "upload",
    value: function upload() {
      this.glPoints = new Float32Array(this.points);
      this.buffer.upload(this.glPoints);
      this.glIndices = new Uint16Array(this.indices);
      this.indexBuffer.upload(this.glIndices);
      this.dirty = false;
    }
    /**
     * Empties all the data
     */

  }, {
    key: "destroy",
    value: function destroy() {
      this.color = null;
      this.points = null;
      this.indices = null;
      this.vao.destroy();
      this.buffer.destroy();
      this.indexBuffer.destroy();
      this.gl = null;
      this.buffer = null;
      this.indexBuffer = null;
      this.glPoints = null;
      this.glIndices = null;
    }
  }]);

  return WebGLGraphicsData;
}();

exports["default"] = WebGLGraphicsData;
//# sourceMappingURL=WebGLGraphicsData.js.map