"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var ObservablePoint = /*#__PURE__*/function () {
  function ObservablePoint(cb, scope) {
    var x = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
    var y = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;

    _classCallCheck(this, ObservablePoint);

    this._x = x;
    this._y = y;
    this.cb = cb;
    this.scope = scope;
  }
  /**
   * Creates a clone of this point.
   * The callback and scope params can be overidden otherwise they will default
   * to the clone object's values.
   *
   * @override
   * @param {Function} [cb=null] - callback when changed
   * @param {object} [scope=null] - owner of callback
   * @return {PISI.ObservablePoint} a copy of the point
   */


  _createClass(ObservablePoint, [{
    key: "clone",
    value: function clone() {
      var cb = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      var _cb = cb || this.cb;

      var _scope = scope || this.scope;

      return new ObservablePoint(_cb, _scope, this._x, this._y);
    }
    /**
     * Sets the point to a new x and y position.
     * If y is omitted, both x and y will be set to x.
     *
     * @param {number} [x=0] - position of the point on the x axis
     * @param {number} [y=0] - position of the point on the y axis
     */

  }, {
    key: "set",
    value: function set(x, y) {
      var _x = x || 0;

      var _y = y || (y !== 0 ? _x : 0);

      if (this._x !== _x || this._y !== _y) {
        this._x = _x;
        this._y = _y;
        this.cb.call(this.scope);
      }
    }
    /**
     * Copies the data from another point
     *
     * @param {PISI.Point|PISI.ObservablePoint} point - point to copy from
     */

  }, {
    key: "copy",
    value: function copy(point) {
      if (this._x !== point.x || this._y !== point.y) {
        this._x = point.x;
        this._y = point.y;
        this.cb.call(this.scope);
      }
    }
    /**
     * Returns true if the given point is equal to this point
     *
     * @param {PISI.Point|PISI.ObservablePoint} p - The point to check
     * @returns {boolean} Whether the given point equal to this point
     */

  }, {
    key: "equals",
    value: function equals(p) {
      return p.x === this._x && p.y === this._y;
    }
    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     *
     * @member {number}
     */

  }, {
    key: "x",
    get: function get() {
      return this._x;
    },
    set: function set(value) {
      if (this._x !== value) {
        this._x = value;
        this.cb.call(this.scope);
      }
    }
    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     *
     * @member {number}
     */

  }, {
    key: "y",
    get: function get() {
      return this._y;
    },
    set: function set(value) {
      if (this._y !== value) {
        this._y = value;
        this.cb.call(this.scope);
      }
    }
  }]);

  return ObservablePoint;
}();

exports["default"] = ObservablePoint;
//# sourceMappingURL=ObservablePoint.js.map