"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _CanvasRenderer = _interopRequireDefault(require("../../renderers/canvas/CanvasRenderer"));

var _const = require("../../const");

var _math = require("../../math");

var _Image = _interopRequireDefault(require("../../polyfill/Image"));

var _CanvasTinter = _interopRequireDefault(require("./CanvasTinter"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var canvasRenderWorldTransform = new _math.Matrix();

var CanvasSpriteRenderer = /*#__PURE__*/function () {
  function CanvasSpriteRenderer(renderer) {
    _classCallCheck(this, CanvasSpriteRenderer);

    this.renderer = renderer;
  }

  _createClass(CanvasSpriteRenderer, [{
    key: "render",
    value: function render(sprite) {
      var texture = sprite._texture;
      var renderer = this.renderer;
      var width = texture._frame.width;
      var height = texture._frame.height;
      var wt = sprite.transform.worldTransform;
      var dx = 0;
      var dy = 0;

      if (texture.orig.width <= 0 || texture.orig.height <= 0 || !texture.baseTexture.source) {
        return;
      }

      renderer.setBlendMode(sprite.blendMode); //  Ignore null sources

      if (texture.valid) {
        renderer.context.globalAlpha = sprite.worldAlpha; // If smoothingEnabled is supported and we need to change the smoothing property for sprite texture

        var smoothingEnabled = texture.baseTexture.scaleMode === _const.SCALE_MODES.LINEAR;

        if (renderer.smoothProperty && renderer.context[renderer.smoothProperty] !== smoothingEnabled) {
          renderer.context[renderer.smoothProperty] = smoothingEnabled;
        }

        if (texture.trim) {
          dx = texture.trim.width / 2 + texture.trim.x - sprite.anchor.x * texture.orig.width;
          dy = texture.trim.height / 2 + texture.trim.y - sprite.anchor.y * texture.orig.height;
        } else {
          dx = (0.5 - sprite.anchor.x) * texture.orig.width;
          dy = (0.5 - sprite.anchor.y) * texture.orig.height;
        }

        if (texture.rotate) {
          wt.copy(canvasRenderWorldTransform);
          wt = canvasRenderWorldTransform;

          _math.GroupD8.matrixAppendRotationInv(wt, texture.rotate, dx, dy); // the anchor has already been applied above, so lets set it to zero


          dx = 0;
          dy = 0;
        }

        dx -= width / 2;
        dy -= height / 2; // Allow for pixel rounding

        if (renderer.roundPixels) {
          renderer.context.setTransform(wt.a, wt.b, wt.c, wt.d, wt.tx * renderer.resolution | 0, wt.ty * renderer.resolution | 0);
          dx = dx | 0;
          dy = dy | 0;
        } else {
          renderer.context.setTransform(wt.a, wt.b, wt.c, wt.d, wt.tx * renderer.resolution, wt.ty * renderer.resolution);
        }

        var resolution = texture.baseTexture.resolution;

        if (sprite.tint !== 0xffffff) {
          if (sprite.cachedTint !== sprite.tint || sprite.tintedTexture.tintId !== sprite._texture._updateID) {
            sprite.cachedTint = sprite.tint; // TODO clean up caching - how to clean up the caches?

            sprite.tintedTexture = _CanvasTinter["default"].getTintedTexture(sprite, sprite.tint);
          }

          renderer.context.drawImage(sprite.tintedTexture, 0, 0, width * resolution, height * resolution, dx * renderer.resolution, dy * renderer.resolution, width * renderer.resolution, height * renderer.resolution);
        } else {
          this.adaptedTexture(texture.baseTexture);
          renderer.context.drawImage(texture.baseTexture.source, texture._frame.x * resolution, texture._frame.y * resolution, width * resolution, height * resolution, dx * renderer.resolution, dy * renderer.resolution, width * renderer.resolution, height * renderer.resolution);
        }
      }
    }
  }, {
    key: "adaptedTexture",
    value: function adaptedTexture(baseTexture) {
      if (!baseTexture.source) return;
      if (baseTexture.source.isPSCanvas) return;

      if (this.isImageData(baseTexture.source)) {
        var source = baseTexture.source,
            width = baseTexture.width,
            height = baseTexture.height;
        baseTexture.source = _Image["default"].convertToCanvas(source, width, height);
      }
    }
  }, {
    key: "isImageData",
    value: function isImageData(data) {
      if (!data) return false;
      if (data.width && data.height && data.data && data.data.length) return true;
      return false;
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.renderer = null;
    }
  }]);

  return CanvasSpriteRenderer;
}();

exports["default"] = CanvasSpriteRenderer;

_CanvasRenderer["default"].registerPlugin("sprite", CanvasSpriteRenderer);
//# sourceMappingURL=CanvasSpriteRenderer.js.map