"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _core = require("../core");

var _utils = require("../utils");

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Spritesheet = /*#__PURE__*/function () {
  function Spritesheet(baseTexture, data) {
    var resolutionFilename = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

    _classCallCheck(this, Spritesheet);

    this.baseTexture = baseTexture;
    this.textures = {};
    this.animations = {};
    this.data = data;
    this.resolution = this._updateResolution(resolutionFilename || this.baseTexture.imageUrl);
    this._frames = this.data.frames;
    this._frameKeys = Object.keys(this._frames);
    this._batchIndex = 0;
    this._callback = null;
  }

  _createClass(Spritesheet, [{
    key: "_updateResolution",
    value: function _updateResolution(resolutionFilename) {
      var scale = this.data.meta.scale; // Use a defaultValue of `null` to check if a url-based resolution is set

      var resolution = (0, _utils.getResolutionOfUrl)(resolutionFilename, null); // No resolution found via URL

      if (resolution === null) {
        // Use the scale value or default to 1
        resolution = scale !== undefined ? parseFloat(scale) : 1;
      } // For non-1 resolutions, update baseTexture


      if (resolution !== 1) {
        this.baseTexture.resolution = resolution;
        this.baseTexture.update();
      }

      return resolution;
    }
  }, {
    key: "parse",
    value: function parse(callback) {
      this._batchIndex = 0;
      this._callback = callback;

      if (this._frameKeys.length <= Spritesheet.BATCH_SIZE) {
        this._processFrames(0);

        this._processAnimations();

        this._parseComplete();
      } else {
        this._nextBatch();
      }
    }
  }, {
    key: "_processFrames",
    value: function _processFrames(initialFrameIndex) {
      var frameIndex = initialFrameIndex;
      var maxFrames = Spritesheet.BATCH_SIZE;
      var sourceScale = this.baseTexture.sourceScale;

      while (frameIndex - initialFrameIndex < maxFrames && frameIndex < this._frameKeys.length) {
        var i = this._frameKeys[frameIndex];
        var data = this._frames[i];
        var rect = data.frame;

        if (rect) {
          var frame = null;
          var trim = null;
          var sourceSize = data.trimmed !== false && data.sourceSize ? data.sourceSize : data.frame;
          var orig = new _core.Rectangle(0, 0, Math.floor(sourceSize.w * sourceScale) / this.resolution, Math.floor(sourceSize.h * sourceScale) / this.resolution);

          if (data.rotated) {
            frame = new _core.Rectangle(Math.floor(rect.x * sourceScale) / this.resolution, Math.floor(rect.y * sourceScale) / this.resolution, Math.floor(rect.h * sourceScale) / this.resolution, Math.floor(rect.w * sourceScale) / this.resolution);
          } else {
            frame = new _core.Rectangle(Math.floor(rect.x * sourceScale) / this.resolution, Math.floor(rect.y * sourceScale) / this.resolution, Math.floor(rect.w * sourceScale) / this.resolution, Math.floor(rect.h * sourceScale) / this.resolution);
          } //  Check to see if the sprite is trimmed


          if (data.trimmed !== false && data.spriteSourceSize) {
            trim = new _core.Rectangle(Math.floor(data.spriteSourceSize.x * sourceScale) / this.resolution, Math.floor(data.spriteSourceSize.y * sourceScale) / this.resolution, Math.floor(rect.w * sourceScale) / this.resolution, Math.floor(rect.h * sourceScale) / this.resolution);
          }

          this.textures[i] = new _core.Texture(this.baseTexture, frame, orig, trim, data.rotated ? 2 : 0, data.anchor); // lets also add the frame to pixi's global cache for fromFrame and fromImage functions

          _core.Texture.addToCache(this.textures[i], i);
        }

        frameIndex++;
      }
    }
  }, {
    key: "_processAnimations",
    value: function _processAnimations() {
      var animations = this.data.animations || {};

      for (var animName in animations) {
        this.animations[animName] = [];

        var _iterator = _createForOfIteratorHelper(animations[animName]),
            _step;

        try {
          for (_iterator.s(); !(_step = _iterator.n()).done;) {
            var frameName = _step.value;
            this.animations[animName].push(this.textures[frameName]);
          }
        } catch (err) {
          _iterator.e(err);
        } finally {
          _iterator.f();
        }
      }
    }
  }, {
    key: "_parseComplete",
    value: function _parseComplete() {
      var callback = this._callback;
      this._callback = null;
      this._batchIndex = 0;
      callback.call(this, this.textures);
    }
  }, {
    key: "_nextBatch",
    value: function _nextBatch() {
      var _this = this;

      this._processFrames(this._batchIndex * Spritesheet.BATCH_SIZE);

      this._batchIndex++;
      setTimeout(function () {
        if (_this._batchIndex * Spritesheet.BATCH_SIZE < _this._frameKeys.length) {
          _this._nextBatch();
        } else {
          _this._processAnimations();

          _this._parseComplete();
        }
      }, 0);
    }
  }, {
    key: "destroy",
    value: function destroy() {
      var destroyBase = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

      for (var i in this.textures) {
        this.textures[i].destroy();
      }

      this._frames = null;
      this._frameKeys = null;
      this.data = null;
      this.textures = null;

      if (destroyBase) {
        this.baseTexture.destroy();
      }

      this.baseTexture = null;
    }
  }], [{
    key: "BATCH_SIZE",
    get: function get() {
      return 1000;
    }
  }]);

  return Spritesheet;
}();

exports["default"] = Spritesheet;
//# sourceMappingURL=Spritesheet.js.map