"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _settings = _interopRequireDefault(require("../settings"));

var _const = require("../const");

var _TickerListener = _interopRequireDefault(require("./TickerListener"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Ticker = /*#__PURE__*/function () {
  function Ticker() {
    var _this = this;

    _classCallCheck(this, Ticker);

    this._head = new _TickerListener["default"](null, null, Infinity);
    this._requestId = null;
    this._maxElapsedMS = 100;
    this.autoStart = false;
    this.deltaTime = 1;
    this.elapsedMS = 1 / _settings["default"].TARGET_FPMS;
    this.lastTime = -1;
    this.speed = 1;
    this.started = false;

    this._tick = function (time) {
      _this._requestId = null;

      if (_this.started) {
        // Invoke listeners now
        _this.update(time); // Listener side effects may have modified ticker state.


        if (_this.started && _this._requestId === null && _this._head.next) {
          _this._requestId = requestAnimationFrame(_this._tick);
        }
      }
    };
  }

  _createClass(Ticker, [{
    key: "_requestIfNeeded",
    value: function _requestIfNeeded() {
      if (this._requestId === null && this._head.next) {
        // ensure callbacks get correct delta
        this.lastTime = performance.now();
        this._requestId = requestAnimationFrame(this._tick);
      }
    }
  }, {
    key: "_cancelIfNeeded",
    value: function _cancelIfNeeded() {
      if (this._requestId !== null) {
        cancelAnimationFrame(this._requestId);
        this._requestId = null;
      }
    }
  }, {
    key: "_startIfPossible",
    value: function _startIfPossible() {
      if (this.started) {
        this._requestIfNeeded();
      } else if (this.autoStart) {
        this.start();
      }
    }
  }, {
    key: "add",
    value: function add(fn, context) {
      var priority = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _const.UPDATE_PRIORITY.NORMAL;
      return this._addListener(new _TickerListener["default"](fn, context, priority));
    }
  }, {
    key: "addOnce",
    value: function addOnce(fn, context) {
      var priority = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _const.UPDATE_PRIORITY.NORMAL;
      return this._addListener(new _TickerListener["default"](fn, context, priority, true));
    }
  }, {
    key: "_addListener",
    value: function _addListener(listener) {
      // For attaching to head
      var current = this._head.next;
      var previous = this._head; // Add the first item

      if (!current) {
        listener.connect(previous);
      } else {
        // Go from highest to lowest priority
        while (current) {
          if (listener.priority > current.priority) {
            listener.connect(previous);
            break;
          }

          previous = current;
          current = current.next;
        } // Not yet connected


        if (!listener.previous) {
          listener.connect(previous);
        }
      }

      this._startIfPossible();

      return this;
    }
  }, {
    key: "remove",
    value: function remove(fn, context) {
      var listener = this._head.next;

      while (listener) {
        if (listener.match(fn, context)) {
          listener = listener.destroy();
        } else {
          listener = listener.next;
        }
      }

      if (!this._head.next) {
        this._cancelIfNeeded();
      }

      return this;
    }
  }, {
    key: "start",
    value: function start() {
      if (!this.started) {
        this.started = true;

        this._requestIfNeeded();
      }
    }
  }, {
    key: "stop",
    value: function stop() {
      if (this.started) {
        this.started = false;

        this._cancelIfNeeded();
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.stop();
      var listener = this._head.next;

      while (listener) {
        listener = listener.destroy(true);
      }

      this._head.destroy();

      this._head = null;
    }
  }, {
    key: "update",
    value: function update() {
      var currentTime = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : performance.now();
      var elapsedMS;

      if (currentTime > this.lastTime) {
        // Save uncapped elapsedMS for measurement
        elapsedMS = this.elapsedMS = currentTime - this.lastTime; // cap the milliseconds elapsed used for deltaTime

        if (elapsedMS > this._maxElapsedMS) {
          elapsedMS = this._maxElapsedMS;
        }

        this.deltaTime = elapsedMS * _settings["default"].TARGET_FPMS * this.speed; // Cache a local reference, in-case ticker is destroyed
        // during the emit, we can still check for head.next

        var head = this._head; // Invoke listeners added to internal emitter

        var listener = head.next;

        while (listener) {
          listener = listener.emit(this.deltaTime);
        }

        if (!head.next) {
          this._cancelIfNeeded();
        }
      } else {
        this.deltaTime = this.elapsedMS = 0;
      }

      this.lastTime = currentTime;
    }
  }, {
    key: "FPS",
    get: function get() {
      return 1000 / this.elapsedMS;
    }
  }, {
    key: "minFPS",
    get: function get() {
      return 1000 / this._maxElapsedMS;
    },
    set: function set(fps) {
      // Clamp: 0 to TARGET_FPMS
      var minFPMS = Math.min(Math.max(0, fps) / 1000, _settings["default"].TARGET_FPMS);
      this._maxElapsedMS = 1 / minFPMS;
    }
  }]);

  return Ticker;
}();

exports["default"] = Ticker;
//# sourceMappingURL=Ticker.js.map